import Pyro.util
import Pyro.core
import Numeric
import threading

class simpleSimulation:
    def __init__(self, offset, multiplyer, arraySize,
                 deltaT, steps, nodes):
        self.deltaT = deltaT
        self.steps = steps
        self.nodes = nodes
        self.offset = offset
        self.multiplyer = multiplyer
        self.numbers = Numeric.array(range(arraySize), Numeric.Float)
        self.results = []
        self.bounds = self.findArrayBounds(self.numbers.shape[0],
                                           len(self.nodes))
        self.connectToNodes()
        self.t = 0

    def findArrayBounds(self, arrayLength, numberOfThreads):
        """Split array in nearly equal sized parts.
            Maximum difference between longest and shortest part is 1.
        """
        if numberOfThreads > arrayLength:
            numberOfThreads = arrayLength
        minArrayLength, extra = divmod(arrayLength, numberOfThreads)
        bounds = []
        lower = 0
        oneMore = 0
        for thread in range(numberOfThreads):
            if extra:
                oneMore = 1
            else:
                oneMore = 0
            upper = lower + minArrayLength + oneMore
            bounds.append((lower, upper))
            extra = max(0, extra - 1)
            lower = upper
        return bounds

    def connectToNodes(self):
        Pyro.core.initClient()
        self.pyroNodes = []
        for node in self.nodes:
            self.pyroNodes.append(Pyro.core.getProxyForURI(
                                  'PYRONAME://calculator_%s' %node))

    def runSimulation(self):
        for step in range(self.steps):
            n = 0
            result = Numeric.zeros(self.numbers.shape, Numeric.Float)
            threads = []
            for pyroNode in self.pyroNodes:
                threads.append(CalculatorThread(pyroNode, self.offset,
                               self.multiplyer))
                lower = self.bounds[n][0]
                upper = self.bounds[n][1]
                threads[n].numericArray = self.numbers[lower:upper]
                threads[n].t = self.t
                threads[n].start()
                n += 1
            for thread in threads:
                thread.join()
            n = 0
            for thread in threads:
                lower = self.bounds[n][0]
                upper = self.bounds[n][1]
                result[lower:upper] = thread.result
                n += 1
            self.results.append(result)
            self.t += self.deltaT

class CalculatorThread(threading.Thread):
    def __init__(self, remoteCalculator, offset, multiplyer):
        threading.Thread.__init__(self)
        self.remoteCalculator = remoteCalculator
        self.remoteCalculator.setParameters(offset, multiplyer)
    def run(self, numericArray, t):
        return self.remoteCalculator.calculate(numericArray, t)

if __name__ == '__main__':
    offset = 3
    multiplyer = 5
    arraySize = 7
    deltaT = 0.5
    steps = 5
    s = simpleSimulation(offset, multiplyer, arraySize,
                          deltaT, steps, ['node1', 'node2'])
    s.runSimulation()
    # check results
    t = 0
    for pyroResult in s.results:
        localResult = (offset + Numeric.array(range(arraySize),
                       Numeric.Float) * multiplyer) * t
        t += deltaT
        print 'pyro results:'
        print pyroResult
        print 'local results:'
        print localResult
